/**
 * 
 */
package org.apache.geronimo;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Date;

import javax.servlet.http.HttpServletResponse;

import org.apache.ahc.AsyncHttpClientCallback;
import org.apache.ahc.codec.HttpResponseMessage;

public class Callback implements AsyncHttpClientCallback {
	private boolean timeout = false;
	private boolean closed = false;
	private boolean exception = false;
	private Throwable throwable = null;
  private HttpServletResponse response;
	private HttpResponseMessage message;
	private long time;
	private String sessionId;

	public Callback(HttpServletResponse response, String sessionId) {
		this.response = response;
		this.sessionId = sessionId;
		clear();
	}
	
	/**
	 * @param time
	 */
	public void setTime(long time){
		this.time = time;
	}
	
	
	/* (non-Javadoc)
	 * @see org.apache.ahc.AsyncHttpClientCallback#onResponse(org.apache.ahc.codec.HttpResponseMessage)
	 */
	public void onResponse(HttpResponseMessage msg) {		
    System.out.println("  onResponse()" + " - Status:" + msg.getStatusCode() + " - Length:" + msg.getContentLength() + " " + sessionId);

    if (timeout) {
        System.out.println("  Already responded to timeout " + sessionId);
        return;
    }
    
    response.setStatus(msg.getStatusCode());
    response.setContentType(msg.getContentType());
		if (msg.getContentLength() > 0) {
			response.setContentLength(msg.getContentLength());
			OutputStream out = null;
			try {
				out = response.getOutputStream();
        out.write(msg.getContent());
				out.flush();
				out.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		System.out.println("  Responded to " + sessionId + " in " + (new Date().getTime() - this.time));
	}

	public void onException(Throwable cause) {
		throwable = cause;
		exception = true;
		System.out.println("  Exception by " + sessionId + " in " + (new Date().getTime() - this.time));		
		cause.printStackTrace();		
	}

	public void onClosed() {
      closed = true;
	}

	public void onTimeout() {
		timeout = true;
    try {
			response.setContentType("text/plain");
			response.setStatus(HttpServletResponse.SC_REQUEST_TIMEOUT);
			PrintWriter out = response.getWriter();
			out.println("Request Timed out!");
			out.println(new Date().toString());
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("  Timed out " + sessionId + " in " + (new Date().getTime() - this.time));
	}


	public Throwable getThrowable() {
		return throwable;
	}

	public void clear() {
		closed = false;
		timeout = false;
		exception = false;
		message = null;
	}

	public boolean isClosed() {
		return closed;
	}

	public void setClosed(boolean closed) {
		this.closed = closed;
	}

	public boolean isTimeout() {
		return timeout;
	}

	public boolean isException() {
		return exception;
	}

	public void setException(boolean exception) {
		this.exception = exception;
	}

	public HttpResponseMessage getMessage() {
		return message;
	}

	public void setMessage(HttpResponseMessage message) {
		this.message = message;
	}

}
